SystemOrganization addCategory: #TusLibros!

TestCase subclass: #CartTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CartTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/12/2012 20:24'!
cart
	
	^Cart new.! !

!CartTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/17/2012 21:05'!
catalogedISBN

	^1234.! !

!CartTest methodsFor: 'tests' stamp: 'GuillermoAres 6/12/2012 20:28'!
test01WhenACartIsCreatedThenItIsEmpty

	self assert: self cart isEmpty = true.! !

!CartTest methodsFor: 'tests' stamp: 'GuillermoAres 6/27/2012 00:09'!
test02CanNotAddANegativeAmountOfBooksToACart

	| aCart aCatalogedISBN|
	
	aCart := self cart.
	aCatalogedISBN = self catalogedISBN.

	self should: [ self cart add: -5 booksWithISBN: self catalogedISBN] 
		 raise: Error
		 withExceptionDo: [ :anError | 
								self assert: anError messageText = Cart canNotAddZeroOrLessBooksToCartErrorDescription.
								self assert: aCart isEmpty].
							! !

!CartTest methodsFor: 'tests' stamp: 'GuillermoAres 6/17/2012 21:05'!
test03IfABookIsAddedToTheCartThenItIsNotEmpty

	self deny: (self cart add: self catalogedISBN; isEmpty).! !

!CartTest methodsFor: 'tests' stamp: 'GuillermoAres 6/17/2012 22:25'!
test04Given3BooksWereAddedToTheCartThenTheyAreInTheCartsContent

	self assert: (((self cart add: 3 booksWithISBN: self catalogedISBN; contentAsBag) occurrencesOf: self catalogedISBN) = 3).! !

!CartTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 01:07'!
test05OnlyBooksFromTheCatalogCanBeAddedToTheCart

	| aCart |
	
	aCart := self cart.

	self should: [ aCart add: self uncatalogedISBN] 
		 raise: Error
		 withExceptionDo: [ :anError | 
								self assert: anError messageText = Cart canNotAddUncatalogedISBNToCartErrorDescription.
								self assert: aCart isEmpty].! !

!CartTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/17/2012 21:03'!
uncatalogedISBN

	^4321.! !

TestCase subclass: #CashierTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/17/2012 23:42'!
ISBNOfBookWorth10Dollars

	^10.! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/17/2012 23:41'!
ISBNOfBookWorth25Dollars

	^25.! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 03:43'!
assertSellWasCancelledBy: aCashier

	self assert: aCashier sellsbook entries = 0.
	self assert: aCashier merchantProcessor debitFromCalls = 0.! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/24/2012 21:50'!
booksWorth100Dollars

	| booksWorth100Dollars |

	booksWorth100Dollars := Bag new.
	
	booksWorth100Dollars 
						add: 25;
						add: 25;
						add: 10;
						add: 10;
						add: 10;
						add: 10;
						add: 10.
								
	^booksWorth100Dollars.! !

!CashierTest methodsFor: 'exceptions' stamp: 'GuillermoAres 6/17/2012 22:58'!
canNotCheckoutAnEmptyCart

	^CanNotCheckoutAnEmptyCart.! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/17/2012 23:43'!
cartWithBooksWorth100Dollars

	^Cart new
		add: 2 booksWithISBN: self ISBNOfBookWorth25Dollars;
		add: 5 booksWithISBN: self ISBNOfBookWorth10Dollars.! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/17/2012 22:49'!
cashier

	^Cashier new.! !

!CashierTest methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 02:38'!
cashierWithMerchantProcessorMock: aMerchantProcessorMock 

	^Cashier withMerchantProcessor: aMerchantProcessorMock.! !

!CashierTest methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 01:58'!
cashierWithMerchantProcessorStub: aMerchantProcessorStub 

	^Cashier withMerchantProcessor: aMerchantProcessorStub.! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/18/2012 01:23'!
creditCard

	^CreditCard ownedBy: 'TestCustomer' identifiedBy: '0123456789012345' expiresOn: (MonthOfYear januaryOf: 2015)! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/18/2012 02:24'!
creditCardWithOwnerNameOver30Chars

	^CreditCard ownedBy: 'TestCustomerWithAVeryVeryVeryLooooooongName' identifiedBy: '12345' expiresOn: (MonthOfYear januaryOf: 2020).! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/20/2012 02:41'!
currentMonthOfYear

	^MonthOfYear onMonthNumber: 6 of: 2012.! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/20/2012 01:32'!
emptyCart

	| anEmptyCart |
	
	anEmptyCart := Cart new.
	
	self assert: anEmptyCart isEmpty.
	
	^anEmptyCart.! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/20/2012 02:35'!
expiredCreditCard

	| anExpiredCreditCard |

	anExpiredCreditCard := CreditCard ownedBy: 'TestCustomer' identifiedBy: '12345' expiresOn: (MonthOfYear januaryOf: 2000).
	
	self assert: (anExpiredCreditCard isExpiredOn: self currentMonthOfYear).
	
	^anExpiredCreditCard.! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 21:50'!
prices

	| pricesByISBN |

	pricesByISBN := Dictionary new.
	
	pricesByISBN 
		add: (Association key: 25 value: 25);
		add: (Association key: 10 value: 10).
		
	^pricesByISBN.! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 03:42'!
test01CanNotCheckoutAnEmptyCart
	
	| anEmptyCart aCashier |
	
	anEmptyCart := self emptyCart.
	aCashier := self cashierWithMerchantProcessorStub: 
										(MerchantProcessorMock withDebitFromImplementation: [^TransactionInfo new]).

	
	self should: [ self cashier checkout: anEmptyCart with: self creditCard] 
		 raise: Error
             withExceptionDo: [ :anError |
								self assert: anError messageText  = Cashier cartIsEmptyErrorDescription. 	
								self assert: anEmptyCart isEmpty.
								self assertSellWasCancelledBy: aCashier.].! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 22:37'!
test02CanNotCheckoutACartWithAnExpiredCreditCard

	| aCashier |
	
	aCashier := self cashierWithMerchantProcessorMock: 
													(MerchantProcessorMock withDebitFromImplementation: [ ^TransactionInfo new]).

	self should: [ aCashier checkout: (self cartWithBooksWorth100Dollars) with: self expiredCreditCard] 
		 raise: Error
		 withExceptionDo: [ :anError | 
								self assert: anError messageText  = Cashier creditCardHasExpiredErrorDescription. 
								self assertSellWasCancelledBy: aCashier].! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 22:37'!
test03CanNotCheckoutACartWithACreditCardWithAnOwnerNameOver30Chars

	| aCashier |
	
	aCashier := self cashierWithMerchantProcessorMock: 
													(MerchantProcessorMock withDebitFromImplementation: [ ^TransactionInfo new]).

	self should: [aCashier checkout: (self cartWithBooksWorth100Dollars) with: self creditCardWithOwnerNameOver30Chars] 
		 raise: Error
		 withExceptionDo: [ :anError | 
								self assert: anError messageText  = Cashier creditCardOwnerNameTooLongErrorDescription.
								self assertSellWasCancelledBy: aCashier].! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 22:37'!
test04CheckingOutACartWithBooksWorth100DollarsReturnsATicketFor100Dollars

	| aCashier aCartWithBooksWorth100Dollars |
	
	aCashier := self cashierWithMerchantProcessorStub:  
													(MerchantProcessorStub withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage]).

	 self assert: ((aCashier checkout: self cartWithBooksWorth100Dollars with: self creditCard) = self ticketFor100Dollars).! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 22:41'!
test05CheckingOutACartWithBooksWorth100DollarsCallsTheMerchantProcessor

	| aCashier aCartWithBooksWorth100Dollars aMerchantProcessorMock |
	
	aMerchantProcessorMock := (MerchantProcessorMock withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage]).
	
	aCashier := self cashierWithMerchantProcessorStub:  aMerchantProcessorMock.
													.

	aCashier checkout: self cartWithBooksWorth100Dollars with: self creditCard.
	
	self assert: ( aMerchantProcessorMock debitFromCalls  = 1).! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 23:25'!
test06CheckingOutACartWithBooksWorth100DollarsLeavesAnEntryInTheCashiersSellBook

	| aCashier aCartWithBooksWorth100Dollars aMerchantProcessorMock aTicket aSell |
	aMerchantProcessorMock := MerchantProcessorMock
		withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage ].
	aCashier := self cashierWithMerchantProcessorStub: aMerchantProcessorMock.
	
	aMerchantProcessorMock := (MerchantProcessorMock withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage]).
	
	aCashier := self cashierWithMerchantProcessorStub:  aMerchantProcessorMock.
													.

	aTicket := aCashier checkout: self cartWithBooksWorth100Dollars with: self creditCard.
	
	aSell := aCashier sellsbook sellChargedTo: self creditCard ownerName.
	
	self assert: aSell = (Sell of: aTicket chargedTo: self creditCard on: aSell date).
! !

!CashierTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 23:29'!
test07CheckingOutACartWithBooksWorth100DollarsLeavesAnEntryInTheCashiersSellBookWithTheCurrentDate

	| aCashier aCartWithBooksWorth100Dollars aMerchantProcessorMock aTicket aSell |
	aMerchantProcessorMock := MerchantProcessorMock
		withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage ].
	aCashier := self cashierWithMerchantProcessorStub: aMerchantProcessorMock.
	
	aMerchantProcessorMock := (MerchantProcessorMock withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage]).
	
	aCashier := self cashierWithMerchantProcessorStub:  aMerchantProcessorMock.
													.

	aTicket := aCashier checkout: self cartWithBooksWorth100Dollars with: self creditCard.
	
	aSell := aCashier sellsbook sellChargedTo: self creditCard ownerName.
	
	self assert: ((DateAndTime now - aSell date) < 1 minutes).
! !

!CashierTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/24/2012 21:50'!
ticketFor100Dollars

	^Ticket for: self booksWorth100Dollars accordingTo: self prices.! !

TestCase subclass: #CreditCardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:32'!
test01CanCreateACreditCardWithValidOwnerNameNumberAndExpirationDate

	self 
		shouldnt: [ CreditCard ownedBy: self validOwnerName identifiedBy: self validNumber expiresOn: self validExpirationDate ]
		raise: Error 
		! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:33'!
test02CreditCardRemembersTheOwnerName

	self assert: (CreditCard ownedBy: self validOwnerName identifiedBy: self validNumber expiresOn: self validExpirationDate) ownerName = self validOwnerName ! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:34'!
test03CreditCardRemembersTheNumber

	self assert: (CreditCard ownedBy: self validOwnerName identifiedBy: self validNumber expiresOn: self validExpirationDate) number = self validNumber! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:44'!
test04CreditCardRemembersTheExpirationDate

	self assert: (CreditCard ownedBy: self validOwnerName identifiedBy: self validNumber expiresOn: self validExpirationDate) expirationDate = self validExpirationDate ! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:51'!
test05CanNotCreateACreditCardWithEmptyOwnerName

	self 
		should: [ CreditCard ownedBy:'' identifiedBy: self validNumber expiresOn: self validExpirationDate ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText  = CreditCard invalidOwnerNameErrorDescription ]! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:51'!
test06CanNotCreateACreditCardWithBlanksOwnerName

	self 
		should: [ CreditCard ownedBy:'  ' identifiedBy: self validNumber expiresOn: self validExpirationDate ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText  = CreditCard invalidOwnerNameErrorDescription ]! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:53'!
test07OwnerNameGetsTrimmed

	self assert: (CreditCard ownedBy: ' pepe ' identifiedBy:  self validNumber expiresOn: self validExpirationDate ) ownerName = 'pepe'! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 12:03'!
test08ACreditCardIsExpiredIfTheCurrentMonthOfYearIsBiggerThanExpirationDate

	| expirationDate currentMonthOfYear |
	
	expirationDate := MonthOfYear onMonthNumber: 1 of: 2012.
	currentMonthOfYear := MonthOfYear onMonthNumber: 2 of:  2012. 
	self assert: ((CreditCard ownedBy: self validOwnerName identifiedBy:  self validNumber expiresOn: expirationDate ) isExpiredOn: currentMonthOfYear) ! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 12:03'!
test09ACreditCardIsNotExpiredIfTheCurrentMonthOfYearIsEqualToExpirationDate

	self deny: ((CreditCard ownedBy: self validOwnerName identifiedBy:  self validNumber expiresOn: self validExpirationDate ) isExpiredOn: self validExpirationDate ) ! !

!CreditCardTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 12:04'!
test10ACreditCardIsNotExpiredIfTheCurrentMonthOfYearIsLessThanExpirationDate

	| expirationDate currentMonthOfYear |
	
	expirationDate := MonthOfYear onMonthNumber: 1 of: 2012.
	currentMonthOfYear := MonthOfYear onMonthNumber: 12 of:  2011. 
	self deny: ((CreditCard ownedBy: self validOwnerName identifiedBy:  self validNumber expiresOn: expirationDate ) isExpiredOn: currentMonthOfYear) ! !

!CreditCardTest methodsFor: 'test objects' stamp: 'HernanWilkinson 6/7/2012 10:31'!
validExpirationDate
	
	^ MonthOfYear januaryOf: 2012! !

!CreditCardTest methodsFor: 'test objects' stamp: 'HernanWilkinson 6/7/2012 10:30'!
validNumber
	
	 ^ '0123456789012345'! !

!CreditCardTest methodsFor: 'test objects' stamp: 'HernanWilkinson 6/7/2012 10:30'!
validOwnerName

	^ 'Pepe Sanchez'! !

TestCase subclass: #MonthOfYearTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:39'!
test01DifferentObjectsAreEqualIfTheyRepresentTheSameMonthOfYear

	self assert: (MonthOfYear onMonthNumber: 1 of: 2012) = (MonthOfYear onMonthNumber: 1 of: 2012)! !

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:42'!
test02DifferentObjectsAreNotEqualfTheyDoNotRepresentTheSameMonthOfYear

	self deny: (MonthOfYear onMonthNumber: 2 of: 2012) = (MonthOfYear onMonthNumber: 1 of: 2012)! !

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:43'!
test03DifferentObjectsThatAreEqualHaveTheSameHash

	self assert: (MonthOfYear onMonthNumber: 1 of: 2012) hash = (MonthOfYear onMonthNumber: 1 of: 2012) hash! !

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:54'!
test04AMonthOfYearIsLessThanOtherIfTheYearIsLess

	self assert: (MonthOfYear onMonthNumber: 1 of: 2011) < (MonthOfYear onMonthNumber: 1 of: 2012) ! !

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:57'!
test05AMonthOfYearIsLessThanOtherIfTheYearIsEqualButTheMonthIsLess

	self assert: (MonthOfYear onMonthNumber: 1 of: 2011) < (MonthOfYear onMonthNumber: 2 of: 2011) ! !

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:58'!
test06AMonthOfYearIsNotLessThanOtherIfTheyAreEqual

	self deny: (MonthOfYear onMonthNumber: 1 of: 2011) < (MonthOfYear onMonthNumber: 1 of: 2011) ! !

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:58'!
test07AMonthOfYearIsNotLessThanOtherIfTheYearIsBigger

	self deny: (MonthOfYear onMonthNumber: 1 of: 2012) < (MonthOfYear onMonthNumber: 1 of: 2011) ! !

!MonthOfYearTest methodsFor: 'test' stamp: 'HernanWilkinson 6/7/2012 11:59'!
test08AMonthOfYearIsNotLessThanOtherIfTheYearIsEqualButTheMonthIsBigger

	self deny: (MonthOfYear onMonthNumber: 2 of: 2011) < (MonthOfYear onMonthNumber: 1 of: 2011) ! !

TestCase subclass: #NullTicketTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!NullTicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 04:06'!
test01NullTicketsAlwaysReturnFalseWhenTestedForEquality

	 | aNullTicket anotherNullTicket|

	aNullTicket := NullTicket new.
	anotherNullTicket := NullTicket new.

	self deny: aNullTicket = aNullTicket.
	self deny: aNullTicket = anotherNullTicket.
	self deny: aNullTicket = Ticket new.
	self deny: aNullTicket = Object new.! !

TestCase subclass: #SellTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!SellTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/24/2012 23:33'!
booksWorth100Dollars

	| booksWorth100Dollars |

	booksWorth100Dollars := Bag new.
	
	booksWorth100Dollars 
						add: 25;
						add: 25;
						add: 10;
						add: 10;
						add: 10;
						add: 10;
						add: 10.
								
	^booksWorth100Dollars.! !

!SellTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/24/2012 23:34'!
creditCard

	^CreditCard ownedBy: 'TestCustomer' identifiedBy: '0123456789012345' expiresOn: (MonthOfYear januaryOf: 2015)! !

!SellTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 23:34'!
prices

	| pricesByISBN |

	pricesByISBN := Dictionary new.
	
	pricesByISBN 
		add: (Association key: 25 value: 25);
		add: (Association key: 10 value: 10).
		
	^pricesByISBN.! !

!SellTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 23:33'!
test01ASellRemembersItsTicket

	| aTicket aSell |
	
	aTicket := Ticket for: self booksWorth100Dollars accordingTo: self prices.
	
	aSell := Sell of: aTicket chargedTo: self creditCard.
	
	self assert: aSell ticket = aTicket.! !

!SellTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 23:35'!
test02ASellRemembersItsDate

	| aTicket aSell aDate |
	
	aTicket := Ticket for: self booksWorth100Dollars accordingTo: self prices.
	
	aDate := DateAndTime now.
	
	aSell := Sell of: aTicket chargedTo: self creditCard on: aDate.
	
	self assert: aSell date = aDate.! !

!SellTest methodsFor: 'tests' stamp: 'GuillermoAres 6/24/2012 23:36'!
test03ASellRemembersItsCreditCard

	| aTicket aSell |
	
	aTicket := Ticket for: self booksWorth100Dollars accordingTo: self prices.
	
	aSell := Sell of: aTicket chargedTo: self creditCard.
	
	self assert: aSell creditCard = self creditCard.! !

!SellTest methodsFor: 'tests' stamp: 'GuillermoAres 6/27/2012 03:25'!
test04CanNotCreateASellWithANullTicket

	| aNullTicket aSell |
	
	aNullTicket := NullTicket new.
	
	self should: [Sell of: aNullTicket chargedTo: self creditCard]
		 raise: Error
		 withExceptionDo: [ :anError |
					self assert: anError messageText = Sell canNotCreateSellWithNullTicketErrorMessage].
! !

TestCase subclass: #TicketTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TicketTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/20/2012 05:26'!
booksWorth100Dollars

	| booksWorth100Dollars |

	booksWorth100Dollars := Bag new.
	
	booksWorth100Dollars 
						add: 25;
						add: 25;
						add: 10;
						add: 10;
						add: 10;
						add: 10;
						add: 10.
								
	^booksWorth100Dollars.! !

!TicketTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/20/2012 05:03'!
booksWorth50Dollars

	| booksWorth50Dollars|

	booksWorth50Dollars := Bag new.

	booksWorth50Dollars 
							add: 25;
							add: 25.
							
	^booksWorth50Dollars.! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 05:30'!
incompletePrices

	| pricesByISBN |

	pricesByISBN := Dictionary new.
	
	pricesByISBN 
		add: (Association key: 25 value: 25).
		
	^pricesByISBN.! !

!TicketTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/20/2012 05:53'!
noBooks

	^Bag new.! !

!TicketTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/20/2012 05:07'!
otherPrices

	| pricesByISBN |

	pricesByISBN := Dictionary new.
	
	pricesByISBN 
		add: (Association key: 25 value: 15);
		add: (Association key: 10 value: 100).
		
	^pricesByISBN.! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 04:57'!
prices

	| pricesByISBN |

	pricesByISBN := Dictionary new.
	
	pricesByISBN 
		add: (Association key: 25 value: 25);
		add: (Association key: 10 value: 10).
		
	^pricesByISBN.! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 05:10'!
test01ATicketRemembersItsPricesInForceAtSellTime
	
	self assert: (Ticket for: self booksWorth100Dollars accordingTo: self prices) pricesInForceAtSellTime  = self prices.! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 05:26'!
test02ATicketRemembersItsMercancy
	
	| booksWorth100Dollars |
	
	booksWorth100Dollars := self booksWorth100Dollars.
	
	self assert: (Ticket for: booksWorth100Dollars accordingTo: self prices) soldItems = booksWorth100Dollars.! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 05:32'!
test03CanNotCreateTicketForUntaggedBooks
	
	self should: [Ticket for: self booksWorth100Dollars accordingTo: self incompletePrices]
		 raise: Error
		 withExceptionDo: [ :anError | 
								self assert: anError messageText = Ticket canNotCreateTicketForUntaggedBooksErrorDescription.
								self deny: (self booksWorth100Dollars allSatisfy: [ :aBook | self incompletePrices includesKey: aBook]).]! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 05:55'!
test04CanNotCreateTicketWithoutMercancy
	
	| noBooks |
	
	noBooks := self noBooks.
	
	self should: [Ticket for: noBooks accordingTo: self prices] 
		   raise: Error
		   withExceptionDo: [ :anError | 
								self assert: anError messageText = Ticket canNotCreateTicketWithoutMercancyErrorDescription.
								self assert: noBooks size = 0.]! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 05:35'!
test05ATicketTotalEqualsTheSumOfEachOfItsItemsPrices
	
	self assert: (Ticket for: self booksWorth100Dollars accordingTo: self prices) total = 100.! !

!TicketTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 05:47'!
test06TwoTicketsAreEqualIfTheyHaveTheSameMercancyAndPrices
	
	| equalTicket1 equalTicket2 ticketWithDifferentMercancy ticketWithDifferentPrices |
	
	equalTicket1 := Ticket for: self booksWorth100Dollars accordingTo: self prices.
	equalTicket2 := Ticket for: self booksWorth100Dollars accordingTo: self prices.
	
	self assert: equalTicket1 = equalTicket2.
	
	ticketWithDifferentMercancy := Ticket for: self booksWorth50Dollars accordingTo: self prices.
	
	self deny: equalTicket1 = ticketWithDifferentMercancy.
	
	ticketWithDifferentPrices := Ticket for: self booksWorth100Dollars accordingTo: self otherPrices.
	
	self deny: equalTicket1 = ticketWithDifferentPrices.! !

TestCase subclass: #TransactionInfoTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TransactionInfoTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 03:54'!
test01ATransactionInfoRemembersItsStatus

	self assert: (TransactionInfo withSuccessfulStatusAndMessage: '') wasSuccessful.
	self deny: (TransactionInfo withUnsuccessfulStatusAndMessage: '') wasSuccessful.! !

!TransactionInfoTest methodsFor: 'tests' stamp: 'GuillermoAres 6/20/2012 03:57'!
test02ATransactionInfoRemembersItsMessage

	self assert: (TransactionInfo withSuccessfulStatusAndMessage: 'test message') message = 'test message'.! !

TestCase subclass: #UserInterfaceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/26/2012 01:32'!
bookISBN

	^10.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/26/2012 01:33'!
bookISBN2

	^20.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/26/2012 01:39'!
bookISBN3

	^30.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/27/2012 00:00'!
bookISBNNotInCatalog

	^-1.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/25/2012 03:49'!
clientId
	
	^1234! !

!UserInterfaceTest methodsFor: 'utils' stamp: 'GuillermoAres 6/26/2012 23:58'!
formatAsResponse: aString 

	^(aString copyReplaceAll: ' ' with: '_') asLowercase.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/26/2012 02:13'!
invalidCartId

	^-1.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/26/2012 23:08'!
invalidClientId

	^-1.! !

!UserInterfaceTest methodsFor: 'utils' stamp: 'GuillermoAres 6/26/2012 01:29'!
parseCreateCartResponse: aCreateCartResponseAsString 
	
	^(aCreateCartResponseAsString subStrings: '|') at: 2.! !

!UserInterfaceTest methodsFor: 'accessing' stamp: 'GuillermoAres 6/25/2012 03:50'!
password

	^ 'password'.! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:01'!
test01WhenAUserCreatesACartWithAValidUserAndAValidPasswordASuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface |

	aUserInterface := self userInterfaceForTesting.

	aResponse := aUserInterface createCartFor: self clientId withPassword: self password.
	
	self assert: (aResponse beginsWithEmpty: '0|' caseSensitive: false).
	self assert: (aResponse size > 2).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:04'!
test02WhenAUserCreatesACartWithAnInvalidUserAnUnsuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface |

	aUserInterface := self userInterfaceForTesting.

	aResponse := aUserInterface createCartFor: self invalidClientId withPassword: self password.
	
	self assert: (aResponse = ('1|', (UserInterface invalidClientIdErrorMessage))).
	! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:19'!
test03WhenAUserCreatesACartWithAValidUserAndAWrongPasswordAnUnsuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface |

	aUserInterface := self userInterfaceForTesting.

	aResponse := aUserInterface createCartFor: self clientId withPassword: self wrongPassword.
	
	self assert: (aResponse = ('1|', (UserInterface wrongPasswordErrorMessage))).
	! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:45'!
test04WhenAUserAddsBooksFromtheCatalogToAValidCartASuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface aCreateCartResponse aCartId |

	aUserInterface := self userInterfaceForTesting.

	aCreateCartResponse := aUserInterface createCartFor: self clientId withPassword: self password.
	
	aCartId := self parseCreateCartResponse: aCreateCartResponse.
	
	aResponse := aUserInterface add: 1 booksWithISBN: self bookISBN toCart: aCartId.
	aResponse := aUserInterface add: 2 booksWithISBN: self bookISBN2 toCart: aCartId.
	aResponse := aUserInterface add: 3 booksWithISBN: self bookISBN3 toCart: aCartId.
		
	self assert: (aResponse = '0|OK').! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:52'!
test05WhenAUserAddsBooksToAnInvalidCartAnUnsuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface aCreateCartResponse aCartId |

	aUserInterface := self userInterfaceForTesting.
	
	aResponse := aUserInterface add: 1 booksWithISBN: self bookISBN toCart: self invalidCartId.
		
	self assert: (aResponse = ('1|', UserInterface invalidCartIdErrorMessage)).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:59'!
test06WhenAUserAddsBooksNotIncludedInTheCatalogToAValidCartAnUnsuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface aCreateCartResponse aCartId |

	aUserInterface := self userInterfaceForTesting.
	
	aCreateCartResponse := aUserInterface createCartFor: self clientId withPassword: self password.
	
	aCartId := self parseCreateCartResponse: aCreateCartResponse.
	
	aResponse := aUserInterface add: 1 booksWithISBN: self bookISBNNotInCatalog toCart: aCartId.
		
	self assert: (aResponse = ('1|', (self formatAsResponse: Cart canNotAddUncatalogedISBNToCartErrorDescription))).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/27/2012 00:08'!
test07WhenAUserAddsANegativeQuantityOfBooksToAValidCartAnUnsuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface aCreateCartResponse aCartId |

	aUserInterface := self userInterfaceForTesting.
	
	aCreateCartResponse := aUserInterface createCartFor: self clientId withPassword: self password.
	
	aCartId := self parseCreateCartResponse: aCreateCartResponse.
	
	aResponse := aUserInterface add: -1 booksWithISBN: self bookISBN toCart: aCartId.
		
	self assert: (aResponse = ('1|', (self formatAsResponse: Cart canNotAddZeroOrLessBooksToCartErrorDescription))).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/27/2012 00:10'!
test08WhenAUserAddsZeroBooksToAValidCartAnUnsuccessfulResponseIsReturnedWithAnAppropiateMessage

	| aResponse aUserInterface aCreateCartResponse aCartId |

	aUserInterface := self userInterfaceForTesting.
	
	aCreateCartResponse := aUserInterface createCartFor: self clientId withPassword: self password.
	
	aCartId := self parseCreateCartResponse: aCreateCartResponse.
	
	aResponse := aUserInterface add: 0 booksWithISBN: self bookISBN toCart: aCartId.
		
	self assert: (aResponse = ('1|', (self formatAsResponse: Cart canNotAddZeroOrLessBooksToCartErrorDescription))).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/27/2012 00:48'!
test09WhenAUserAddsBooksToACartCreatedMoreThan30MinutesBeforeAnUnsuccessfulResponseIsReturnedWithAPertinentMessage

	| aResponse aUserInterface aCreateCartResponse aCartId now|

	now := DateAndTime now.

	aUserInterface := self userInterfaceForTestingWithSystemClock: 
															(Stub implementing: #now as: [now]).	
	
	aCreateCartResponse := aUserInterface createCartFor: self clientId withPassword: self password.
	
	aCartId := self parseCreateCartResponse: aCreateCartResponse.
	
	now := now + 30 minutes + 1 second.
	
	aResponse := aUserInterface add: 1 booksWithISBN: self bookISBN toCart: aCartId.
		
	self assert: (aResponse = ('1|', (self formatAsResponse: UserInterface cartHasExpiredErrorMessage))).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:51'!
test10ListCartWithAValidCartIdReturnsASuccessfulResponseWithAPertinentMessage

	|  aUserInterface aCartId aListCartResponse |

	aUserInterface := self userInterfaceForTesting.

	aCartId := self parseCreateCartResponse: (aUserInterface createCartFor: self clientId withPassword: self password).
	
	aUserInterface add: 3 booksWithISBN: self bookISBN toCart: aCartId.
	aUserInterface add: 2 booksWithISBN: self bookISBN2 toCart: aCartId.
	aUserInterface add: 1 booksWithISBN: self bookISBN3 toCart: aCartId.
		
	aListCartResponse := aUserInterface listCart: aCartId.
	
	self deny: (self bookISBN = self bookISBN2) | (self bookISBN = self bookISBN3) | (self bookISBN2 = self bookISBN3).
	self assert: (aListCartResponse beginsWithEmpty: '0' caseSensitive: false ).
	self assert: (aListCartResponse includesSubString: '|30|1').
	self assert: (aListCartResponse includesSubString: '|20|2').
	self assert: (aListCartResponse includesSubString: '|10|3').
	self assert: (aListCartResponse size = 16).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/26/2012 23:51'!
test11ListCartWithAValidEmptyCartIdReturnsASuccessfulResponseWithAPertinentMessage

	|  aUserInterface aCartId aListCartResponse |

	aUserInterface := self userInterfaceForTesting.

	aCartId := self parseCreateCartResponse: (aUserInterface createCartFor: self clientId withPassword: self password).
		
	aListCartResponse := aUserInterface listCart: aCartId.
	
	self deny: (self bookISBN = self bookISBN2) | (self bookISBN = self bookISBN3) | (self bookISBN2 = self bookISBN3).
	self assert: (aListCartResponse beginsWithEmpty: '0' caseSensitive: false ).
	self assert: (aListCartResponse size = 1).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/27/2012 00:12'!
test12ListCartWithAValidCartIdReturnsASuccessfulResponseWithAPertinentMessage

	|  aUserInterface aCartId aListCartResponse |

	aUserInterface := self userInterfaceForTesting.
		
	aListCartResponse := aUserInterface listCart: self invalidCartId.

	self assert: (aListCartResponse = ('1|', UserInterface invalidCartIdErrorMessage)).! !

!UserInterfaceTest methodsFor: 'tests' stamp: 'GuillermoAres 6/27/2012 01:16'!
test13ListCartWithAnExpiredCartReturnsAnUnsuccessfulResponseWithPertinentMessage

	|  aUserInterface aCartId aListCartResponse now aCreateCartResponse |

	now := DateAndTime now.

	aUserInterface := self userInterfaceForTestingWithSystemClock: 
																(Stub implementing: #now as: [now]).
		
	aCreateCartResponse := aUserInterface createCartFor: self clientId withPassword: self password.
	
	aCartId := self parseCreateCartResponse: aCreateCartResponse.
																
	now := now + 30 minutes + 1 second.
		
	aListCartResponse := aUserInterface listCart:  aCartId.

	self assert: (aListCartResponse = ('1|', UserInterface cartHasExpiredErrorMessage)).! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/25/2012 04:03'!
userInterfaceForTesting

	| aUserInterface |

	^aUserInterface := UserInterface forTesting.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/27/2012 00:51'!
userInterfaceForTestingWithSystemClock: aSystemClock

	^UserInterface forTestingWithSystemClock: aSystemClock.! !

!UserInterfaceTest methodsFor: 'domain entities' stamp: 'GuillermoAres 6/26/2012 23:13'!
wrongPassword

	^-1.! !

Magnitude subclass: #MonthOfYear
	instanceVariableNames: 'monthNumber yearNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MonthOfYear class methodsFor: 'initialization' stamp: 'GuillermoAres 6/18/2012 01:47'!
current
	
	| today |
	
	today := Date today.
	
	^self onMonthNumber: today month index of: today year.! !

!MonthOfYear class methodsFor: 'initialization' stamp: 'HernanWilkinson 6/7/2012 10:32'!
januaryOf: aYearNumber

	^ self onMonthNumber: 1 of: aYearNumber  
	
	! !

!MonthOfYear class methodsFor: 'initialization' stamp: 'HernanWilkinson 6/7/2012 10:32'!
onMonthNumber: aMonthNumber of: aYearNumber

	^ self new initializeOnMonthNumber: aMonthNumber of: aYearNumber! !

!MonthOfYear methodsFor: 'comparing' stamp: 'HernanWilkinson 6/7/2012 11:56'!
< aMonthOfYear

	^ yearNumber < aMonthOfYear yearNumber  or: [ yearNumber = aMonthOfYear yearNumber and: monthNumber < aMonthOfYear monthNumber ]! !

!MonthOfYear methodsFor: 'comparing' stamp: 'HernanWilkinson 6/7/2012 11:40'!
= anObject

	^ (anObject isKindOf: self class) and: [ monthNumber = anObject monthNumber and: [ yearNumber = anObject yearNumber ]]! !

!MonthOfYear methodsFor: 'comparing' stamp: 'HernanWilkinson 6/7/2012 11:44'!
hash

	^ monthNumber hash bitXor: yearNumber hash! !

!MonthOfYear methodsFor: 'initialization' stamp: 'HernanWilkinson 6/7/2012 11:42'!
initializeOnMonthNumber: aMonthNumber of: aYearNumber

	monthNumber := aMonthNumber.
	yearNumber := aYearNumber! !

!MonthOfYear methodsFor: 'month' stamp: 'HernanWilkinson 6/7/2012 11:40'!
monthNumber
	
	^ monthNumber! !

!MonthOfYear methodsFor: 'year' stamp: 'HernanWilkinson 6/7/2012 11:41'!
yearNumber
	
	^ yearNumber! !

Object subclass: #Cart
	instanceVariableNames: 'content catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'error descriptions' stamp: 'GuillermoAres 6/18/2012 02:40'!
canNotAddUncatalogedISBNToCartErrorDescription

	^ 'Can not add a book with uncataloged ISBN to the cart.'! !

!Cart class methodsFor: 'error descriptions' stamp: 'GuillermoAres 6/27/2012 00:08'!
canNotAddZeroOrLessBooksToCartErrorDescription

	^ 'Can not add a negative amount of books to the cart'.! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'GuillermoAres 6/26/2012 01:38'!
withCatalog: isbns

	^self new withCatalog: isbns.! !

!Cart methodsFor: 'adding' stamp: 'GuillermoAres 6/18/2012 02:41'!
add: anISBN 

	self checkIfIsCataloged: anISBN.
	content add: anISBN.! !

!Cart methodsFor: 'adding' stamp: 'GuillermoAres 6/27/2012 00:08'!
add: anAmount booksWithISBN: anISBN
	
	(anAmount <= 0) ifTrue: [^self error: Cart canNotAddZeroOrLessBooksToCartErrorDescription ].
	self checkIfIsCataloged: anISBN.
	content add: anISBN withOccurrences: anAmount.! !

!Cart methodsFor: 'assertions' stamp: 'GuillermoAres 6/18/2012 02:39'!
checkIfIsCataloged: anISBN

	(catalog includes: anISBN) ifFalse: [ ^self error: Cart canNotAddUncatalogedISBNToCartErrorDescription].! !

!Cart methodsFor: 'private' stamp: 'GuillermoAres 6/17/2012 20:38'!
content

	^ content! !

!Cart methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 02:04'!
contentAsBag

	^ content copy.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'GuillermoAres 6/26/2012 01:52'!
contentAsDictionary

	^content valuesAndCounts.! !

!Cart methodsFor: 'initialization' stamp: 'GuillermoAres 6/18/2012 02:43'!
initialize

	content := Bag new.
	catalog := Set new.
	catalog add: 1234;
			add: 25;
			add: 10.! !

!Cart methodsFor: 'testing' stamp: 'GuillermoAres 6/17/2012 20:52'!
isEmpty
	
	^self content size = 0.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'GuillermoAres 6/24/2012 22:27'!
prices
	self shouldBeImplemented.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'GuillermoAres 6/26/2012 01:39'!
withCatalog: isbns 
	
	catalog := isbns.! !

Object subclass: #Cashier
	instanceVariableNames: 'content merchantProcessor pricesByISBN sellsBook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'error descriptions' stamp: 'GuillermoAres 6/18/2012 02:31'!
cartIsEmptyErrorDescription

	^ 'The cart is empty.'! !

!Cashier class methodsFor: 'error descriptions' stamp: 'GuillermoAres 6/18/2012 02:14'!
creditCardHasExpiredErrorDescription

	^ 'The credit card has expired.'! !

!Cashier class methodsFor: 'error descriptions' stamp: 'GuillermoAres 6/18/2012 02:19'!
creditCardOwnerNameTooLongErrorDescription

	^ 'The credit card owner name is too long.'! !

!Cashier class methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 01:58'!
withMerchantProcessor: aMerchantProcessorStub 

	^self new withMerchantProcessor: aMerchantProcessorStub.! !

!Cashier methodsFor: 'assertions' stamp: 'GuillermoAres 6/18/2012 02:32'!
checkIfIsEmpty: aCart 

	(aCart isEmpty) ifTrue: [^self error: Cashier cartIsEmptyErrorDescription].! !

!Cashier methodsFor: 'checkouting' stamp: 'GuillermoAres 6/24/2012 23:20'!
checkout: aCart with: aCreditCard

	| finalPrice aTransactionInfo aTicket |
	
	self checkIfIsEmpty: aCart.
	
	finalPrice := 0.
	
	aCart contentAsBag do: [ :anISBN | finalPrice := finalPrice + (pricesByISBN at: anISBN) ].
	self validateCreditCard: aCreditCard.
	
	[ aTransactionInfo := merchantProcessor debit: finalPrice from: aCreditCard ]
		on: Error
		do: [ ^ NullTicket new ].
		
	aTicket := Ticket for: aCart contentAsBag accordingTo: pricesByISBN.
	
	aTransactionInfo wasSuccessful
		ifTrue: [ sellsBook add: (Sell of: aTicket chargedTo: aCreditCard) ]
		ifFalse: [ self error: aTransactionInfo message ].
		
	^ aTicket! !

!Cashier methodsFor: 'assertions' stamp: 'GuillermoAres 6/20/2012 02:08'!
initialize 

	merchantProcessor := MerchantProcessor new.
	pricesByISBN := Dictionary new.
	pricesByISBN 
		add: (Association key: 25 value: 25);
		add: (Association key: 10 value: 10).
	sellsBook := SellsBook new.! !

!Cashier methodsFor: 'checkouting' stamp: 'GuillermoAres 6/20/2012 03:40'!
merchantProcessor

	^merchantProcessor.! !

!Cashier methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 02:08'!
sellsbook

	^sellsBook copy.! !

!Cashier methodsFor: 'assertions' stamp: 'GuillermoAres 6/18/2012 02:34'!
validateCreditCard: aCreditCard 

	(aCreditCard isExpiredOn: MonthOfYear current) ifTrue: [ self error: Cashier creditCardHasExpiredErrorDescription].
	(aCreditCard ownerName size > 30) ifTrue: [ self error: Cashier creditCardOwnerNameTooLongErrorDescription].! !

!Cashier methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 01:58'!
withMerchantProcessor: aMerchantProcessorStub 

	merchantProcessor := aMerchantProcessorStub.! !

Object subclass: #CreditCard
	instanceVariableNames: 'ownerName number expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'error descriptions' stamp: 'HernanWilkinson 6/7/2012 11:49'!
invalidOwnerNameErrorDescription
	
	^ 'Owner name can not be empty'! !

!CreditCard class methodsFor: 'instance creation' stamp: 'GuillermoAres 6/18/2012 02:09'!
ownedBy: anOwnerName identifiedBy: aNumber expiresOn: anExpirationDate 
	
	| trimmedOwnerName | 
	
	trimmedOwnerName := anOwnerName trimBoth.
	self validateOwnerName: trimmedOwnerName.
	
	^ self new initializeOwnedBy: trimmedOwnerName identifiedBy: aNumber expiresOn: anExpirationDate 
! !

!CreditCard class methodsFor: 'assertions' stamp: 'HernanWilkinson 6/7/2012 11:48'!
validateOwnerName: trimmedOwnerName

	trimmedOwnerName isEmpty ifTrue: [ self error: self invalidOwnerNameErrorDescription ]! !

!CreditCard methodsFor: 'initialization' stamp: 'GuillermoAres 6/24/2012 23:24'!
= anotherCreditCard

	self == anotherCreditCard ifTrue: [ ^true].
	(self class == anotherCreditCard class) ifFalse: [ ^false].

	^ownerName = anotherCreditCard ownerName
	  and: number = anotherCreditCard number
	  and: expirationDate = anotherCreditCard expirationDate.! !

!CreditCard methodsFor: 'expiration' stamp: 'HernanWilkinson 6/7/2012 11:37'!
expirationDate

	^ expirationDate! !

!CreditCard methodsFor: 'initialization' stamp: 'HernanWilkinson 6/7/2012 11:36'!
initializeOwnedBy: anOwnerName identifiedBy: aNumber expiresOn: anExpirationDate 
	
	ownerName := anOwnerName.
	number := aNumber.
	expirationDate := anExpirationDate! !

!CreditCard methodsFor: 'expiration' stamp: 'HernanWilkinson 6/7/2012 12:02'!
isExpiredOn: aMonthOfYear 
	
	^ aMonthOfYear > expirationDate ! !

!CreditCard methodsFor: 'number' stamp: 'HernanWilkinson 6/7/2012 11:34'!
number
	^ number! !

!CreditCard methodsFor: 'owner' stamp: 'HernanWilkinson 6/7/2012 11:34'!
ownerName
	^ ownerName! !

Object subclass: #MerchantProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MerchantProcessor methodsFor: 'as yet unclassified' stamp: 'GuillermoAres 6/18/2012 03:20'!
debit: anAmount from: aCreditCard

	! !

MerchantProcessor subclass: #MerchantProcessorStub
	instanceVariableNames: 'debitFromImplementation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

MerchantProcessorStub subclass: #MerchantProcessorMock
	instanceVariableNames: 'debitFromCalls'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MerchantProcessorMock methodsFor: 'initialization' stamp: 'GuillermoAres 6/24/2012 22:42'!
debit: anAmount from: aCreditCard

	debitFromCalls := debitFromCalls + 1.

	^super debit: anAmount from: aCreditCard.! !

!MerchantProcessorMock methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 02:15'!
debitFromCalls

	^debitFromCalls.! !

!MerchantProcessorMock methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 02:23'!
initialize

	debitFromCalls := 0.! !

!MerchantProcessorStub class methodsFor: 'instance creation' stamp: 'GuillermoAres 6/20/2012 01:55'!
withDebitFromImplementation: aBlockClosure 

	^self new withDebitFromImplementation: aBlockClosure.! !

!MerchantProcessorStub methodsFor: 'initialization' stamp: 'GuillermoAres 6/24/2012 22:19'!
debit: anAmount from: aCreditCard

	^debitFromImplementation value.! !

!MerchantProcessorStub methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 01:56'!
withDebitFromImplementation: aBlockClosure 

	debitFromImplementation:= aBlockClosure.! !

Object subclass: #Sell
	instanceVariableNames: 'ticket creditCard date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Sell class methodsFor: 'error messages' stamp: 'GuillermoAres 6/27/2012 03:25'!
canNotCreateSellWithNullTicketErrorMessage

	^ 'Can not create a Sell with a NullTicket.'! !

!Sell class methodsFor: 'initialization' stamp: 'GuillermoAres 6/27/2012 03:25'!
of: aTicket chargedTo: aCreditCard

	aTicket ifNull: [self error: Sell canNotCreateSellWithNullTicketErrorMessage].

	^ self new of: aTicket chargedTo: aCreditCard on: DateAndTime now.
! !

!Sell class methodsFor: 'initialization' stamp: 'GuillermoAres 6/24/2012 23:19'!
of: aTicket chargedTo: aCreditCard on: aDate

	^ self new of: aTicket chargedTo: aCreditCard on: aDate.
! !

!Sell methodsFor: 'comparing' stamp: 'GuillermoAres 6/24/2012 23:23'!
= anotherSell

	self == anotherSell ifTrue: [ ^true].
	(self class == anotherSell class) ifFalse: [ ^false].

	^creditCard = anotherSell creditCard
	  and: ticket = anotherSell ticket
	  and: date = anotherSell date.! !

!Sell methodsFor: 'accessing' stamp: 'GuillermoAres 6/24/2012 22:33'!
creditCard

	^ creditCard! !

!Sell methodsFor: 'accessing' stamp: 'GuillermoAres 6/24/2012 23:20'!
date

	^ date! !

!Sell methodsFor: 'initialization' stamp: 'GuillermoAres 6/24/2012 23:19'!
of: aTicket chargedTo: aCreditCard on: aDate 

	ticket := aTicket.
	creditCard := aCreditCard.
	date := aDate.
! !

!Sell methodsFor: 'accessing' stamp: 'GuillermoAres 6/24/2012 22:54'!
ticket

	^ ticket! !

Object subclass: #SellsBook
	instanceVariableNames: 'content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!SellsBook methodsFor: 'adding' stamp: 'GuillermoAres 6/24/2012 22:53'!
add: aSell 

	content add: (Association key: aSell creditCard ownerName value: aSell).! !

!SellsBook methodsFor: 'testing' stamp: 'GuillermoAres 6/20/2012 02:09'!
entries

	^content size.! !

!SellsBook methodsFor: 'initialization' stamp: 'GuillermoAres 6/24/2012 22:49'!
initialize

	content := Dictionary new.! !

!SellsBook methodsFor: 'accessing' stamp: 'GuillermoAres 6/27/2012 03:33'!
sellChargedTo: aClientName 

	^content at: aClientName.! !

Object subclass: #Stub
	instanceVariableNames: 'implementations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Stub class methodsFor: 'initialization' stamp: 'GuillermoAres 6/27/2012 00:49'!
implementing: aSelector as: aBlockClosure 

	^self new implementing: aSelector as: aBlockClosure.! !

!Stub methodsFor: 'system primitives' stamp: 'GuillermoAres 6/27/2012 01:10'!
doesNotUnderstand: aMessage

	^(implementations at: (aMessage selector)) value.! !

!Stub methodsFor: 'initialization' stamp: 'GuillermoAres 6/27/2012 00:50'!
implementing: aSelector as: aBlockClosure

	implementations add: (Association key: aSelector value: aBlockClosure).! !

!Stub methodsFor: 'initialization' stamp: 'GuillermoAres 6/27/2012 00:47'!
initialize 

	implementations := Dictionary new.! !

Object subclass: #SystemClock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!SystemClock methodsFor: 'accessing' stamp: 'GuillermoAres 6/27/2012 01:00'!
now

	^DateAndTime current.! !

Object subclass: #Ticket
	instanceVariableNames: 'soldItems pricesInForceAtSellTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Ticket class
	instanceVariableNames: 'soldItems total'!

Ticket class
	instanceVariableNames: 'soldItems total'!

Ticket subclass: #NullTicket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!NullTicket methodsFor: 'comparing' stamp: 'GuillermoAres 6/20/2012 03:04'!
= anotherTicket

	 ^false.! !

!NullTicket methodsFor: 'testing' stamp: 'GuillermoAres 6/18/2012 03:30'!
hash

	self shouldBeImplemented.! !

!NullTicket methodsFor: 'execution control' stamp: 'GuillermoAres 6/24/2012 23:55'!
ifNull: aBlockClosure 

	aBlockClosure value.! !

!NullTicket methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 05:44'!
pricesInForceAtSellTime

	self shouldBeImplemented.! !

!NullTicket methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 05:44'!
soldItems

	self shouldBeImplemented.! !

!NullTicket methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 05:44'!
total

	self shouldBeImplemented.! !

!Ticket class methodsFor: 'error descriptions' stamp: 'GuillermoAres 6/20/2012 05:23'!
canNotCreateTicketForUntaggedBooksErrorDescription

	^ 'Can not create a ticket for untagged books.'! !

!Ticket class methodsFor: 'error descriptions' stamp: 'GuillermoAres 6/20/2012 05:54'!
canNotCreateTicketWithoutMercancyErrorDescription

	^ 'Can not create a Ticket without mercancy.'! !

!Ticket class methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 05:53'!
for: items accordingTo: prices 

	(items size = 0) ifTrue: [ ^self error: Ticket canNotCreateTicketWithoutMercancyErrorDescription.].
	(items allSatisfy: [ :item | prices includesKey: item]) ifFalse: [ ^self error: Ticket canNotCreateTicketForUntaggedBooksErrorDescription.].

	^Ticket new for: items accordingTo: prices.! !

!Ticket methodsFor: 'equality' stamp: 'GuillermoAres 6/24/2012 23:11'!
= anotherTicket

	self == anotherTicket ifTrue: [ ^true].
	(self class == anotherTicket class) ifFalse: [ ^false].

	^soldItems = anotherTicket soldItems 
	  and: pricesInForceAtSellTime = anotherTicket pricesInForceAtSellTime.! !

!Ticket methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 05:02'!
for: items accordingTo: prices 

	soldItems := items.
	pricesInForceAtSellTime := prices.! !

!Ticket methodsFor: 'equality' stamp: 'GuillermoAres 6/18/2012 00:51'!
hash

	^self amount.! !

!Ticket methodsFor: 'execution control' stamp: 'GuillermoAres 6/24/2012 23:55'!
ifNull: aBlockClosure 

! !

!Ticket methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 05:07'!
pricesInForceAtSellTime

	^ pricesInForceAtSellTime! !

!Ticket methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 04:36'!
soldItems

	^ soldItems! !

!Ticket methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 05:43'!
total

	^ (soldItems collect: [ :anItem | pricesInForceAtSellTime at: anItem ]) sum. ! !

Object subclass: #TransactionInfo
	instanceVariableNames: 'wasSuccessful message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TransactionInfo class methodsFor: 'messages' stamp: 'GuillermoAres 6/20/2012 03:20'!
wasSuccessfulMessage

	^ 'The transaction was successful.'.! !

!TransactionInfo class methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 03:20'!
withSuccessfulStatusAndMessage: aMessage

	^ TransactionInfo new withSuccessfulStatusAndMessage: aMessage.! !

!TransactionInfo class methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 03:55'!
withUnsuccessfulStatusAndMessage: aMessage

	^self new withUnsuccessfulStatusAndMessage: aMessage.! !

!TransactionInfo methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 03:57'!
message

	^ message! !

!TransactionInfo methodsFor: 'accessing' stamp: 'GuillermoAres 6/20/2012 03:14'!
wasSuccessful

	^wasSuccessful.! !

!TransactionInfo methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 03:19'!
withSuccessfulStatusAndMessage: aMessage.

	wasSuccessful := true.
	message := aMessage.! !

!TransactionInfo methodsFor: 'initialization' stamp: 'GuillermoAres 6/20/2012 03:56'!
withUnsuccessfulStatusAndMessage: aMessage 

	wasSuccessful := false.
	message := message.! !

Object subclass: #UserInterface
	instanceVariableNames: 'clients carts cashier cartsCreated catalog systemClock cartCreationDateAndTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!UserInterface class methodsFor: 'error messages' stamp: 'GuillermoAres 6/27/2012 00:53'!
cartHasExpiredErrorMessage

	^ 'cart_has_expired'.! !

!UserInterface class methodsFor: 'initialization' stamp: 'GuillermoAres 6/25/2012 04:04'!
forTesting

	^self new forTesting.! !

!UserInterface class methodsFor: 'initialization' stamp: 'GuillermoAres 6/27/2012 00:52'!
forTestingWithSystemClock: aSystemClock 

	^self new forTestingWithSystemClock: aSystemClock.! !

!UserInterface class methodsFor: 'error messages' stamp: 'GuillermoAres 6/26/2012 02:13'!
invalidCartIdErrorMessage

	^'invalid_cart_id'.! !

!UserInterface class methodsFor: 'error messages' stamp: 'GuillermoAres 6/26/2012 23:08'!
invalidClientIdErrorMessage

	^ 'invalid_client_id'.! !

!UserInterface class methodsFor: 'error messages' stamp: 'GuillermoAres 6/26/2012 02:23'!
unexpectedErrorMessage

	^'internal_server_error'.! !

!UserInterface class methodsFor: 'error messages' stamp: 'GuillermoAres 6/26/2012 23:19'!
wrongPasswordErrorMessage

	^'wrong_password'.! !

!UserInterface methodsFor: 'REST interface' stamp: 'GuillermoAres 6/27/2012 11:25'!
add: aQuantity booksWithISBN: aBookISBN toCart: aCartId

	| aCart aContext |

	aContext := thisContext.

	^self try: [
		
		aCart := self retrieveCart: aCartId onErrorReturnFrom: aContext.
	
		[aCart add: aQuantity booksWithISBN: aBookISBN]
													on: Error
													do: [ :anError | ^'1|', (self formatAsResponse: anError messageText) ].
	
		^'0|OK'.
	].! !

!UserInterface methodsFor: 'REST interface' stamp: 'GuillermoAres 6/27/2012 11:25'!
createCartFor: aClientId withPassword: aPassword

	| aCart aCartKey aContext |

	aContext := thisContext.

	^self try: [
		
	self validateClient: aClientId withPassword: aPassword onErrorReturnFrom: aContext.
	
	aCart := Cart withCatalog: catalog.
	
	aCartKey := self keyForCart: aCart.
	
	carts add: (Association key: aCartKey value: aCart).
	cartCreationDateAndTime add: (Association key: aCartKey value: systemClock now).
	
	^'0|', aCartKey asString.].! !

!UserInterface methodsFor: 'initialization' stamp: 'GuillermoAres 6/26/2012 01:37'!
forTesting

	cashier := Cashier withMerchantProcessor: 
										(MerchantProcessorMock withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage]).
	clients add: (Association key: 1234 value: 'password').
	catalog add: 10;
			add: 20;
			add: 30;
			add: 40.
			! !

!UserInterface methodsFor: 'initialization' stamp: 'GuillermoAres 6/27/2012 00:23'!
forTestingWithSystemClock: aSystemClock

	cashier := Cashier withMerchantProcessor: 
										(MerchantProcessorMock withDebitFromImplementation: [ TransactionInfo withSuccessfulStatusAndMessage: TransactionInfo wasSuccessfulMessage]).
										
	clients add: (Association key: 1234 value: 'password').
	catalog add: 10;
			add: 20;
			add: 30;
			add: 40.
			
	systemClock := aSystemClock.			! !

!UserInterface methodsFor: 'response formatting' stamp: 'GuillermoAres 6/26/2012 23:59'!
formatAsResponse: aString 
	
	^(aString copyReplaceAll: ' ' with: '_') asLowercase.! !

!UserInterface methodsFor: 'initialization' stamp: 'GuillermoAres 6/27/2012 00:22'!
initialize

	carts := Dictionary new.
	cartCreationDateAndTime := Dictionary new.
	clients := Dictionary new.
	cartsCreated := 0.
	catalog := Set new.
	systemClock := SystemClock new.! !

!UserInterface methodsFor: 'key generators' stamp: 'GuillermoAres 6/27/2012 01:24'!
keyForCart: aCart 

	cartsCreated := cartsCreated + 1.

	^cartsCreated.! !

!UserInterface methodsFor: 'REST interface' stamp: 'GuillermoAres 6/27/2012 11:26'!
listCart: aCartId

	| aCart responseParameters aContext |
	
	aContext := thisContext.
	
	^self try: [
		
		aCart := self retrieveCart: aCartId onErrorReturnFrom: aContext.
		
		responseParameters := ''.
		aCart contentAsDictionary associationsDo: [ :bookISBNAndQuantity | 
																	responseParameters := responseParameters, '|', bookISBNAndQuantity key asString, '|', bookISBNAndQuantity value asString].
		^'0', responseParameters.
	].! !

!UserInterface methodsFor: 'exception handling' stamp: 'GuillermoAres 6/27/2012 03:14'!
retrieveCart: aCartId onErrorReturnFrom: aContext

	| aCartCreationDateAndTime aCart |
	
	[aCartCreationDateAndTime := cartCreationDateAndTime at: aCartId asInteger.]
																					on: KeyNotFound 
																					do: [ aContext return: ('1|', (UserInterface invalidCartIdErrorMessage)).].
		
	(systemClock now > (aCartCreationDateAndTime + 30 minutes)) 
															ifTrue:  [aContext return: ('1|', (UserInterface cartHasExpiredErrorMessage))].
		
	[aCart := carts at: aCartId asInteger.]
									on: KeyNotFound
									do: [ aContext return: ('1|', (UserInterface invalidCartIdErrorMessage))].
									
	^aCart.! !

!UserInterface methodsFor: 'exception handling' stamp: 'GuillermoAres 6/27/2012 00:00'!
try: aBlockClosure 

	^aBlockClosure
				on: Exception
				do: [ ^'1|', (UserInterface unexpectedErrorMessage)].! !

!UserInterface methodsFor: 'exception handling' stamp: 'GuillermoAres 6/27/2012 03:12'!
validateClient: aClientId withPassword: aPassword onErrorReturnFrom: aContext

	| expectedPassword |
	
	[expectedPassword := clients at: aClientId.]
										on: KeyNotFound 
										do: [ aContext return: ('1|', (UserInterface invalidClientIdErrorMessage))].
	
	(expectedPassword = aPassword) ifFalse:[ aContext return: ('1|', (UserInterface wrongPasswordErrorMessage))].! !
